/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "RemoteRenderingBackend.h"

#include "ArgumentCoders.h"
#include "BufferIdentifierSet.h"
#include "Decoder.h"
#include "FilterReference.h"
#include "FontReference.h"
#include "HandleMessage.h"
#include "ImageBufferBackendHandle.h"
#include "RemoteRenderingBackendMessages.h"
#include "RenderingUpdateID.h"
#include "ShareableBitmap.h"
#include "SharedMemory.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/AlphaPremultiplication.h>
#include <WebCore/ColorSpace.h>
#include <WebCore/FloatSize.h>
#include <WebCore/ImageBufferBackend.h>
#include <WebCore/IntPoint.h>
#include <WebCore/IntRect.h>
#include <WebCore/PixelBuffer.h>
#include <WebCore/PixelBufferFormat.h>
#include <WebCore/RenderingMode.h>
#include <WebCore/RenderingResourceIdentifier.h>
#include <optional>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebKit {

void RemoteRenderingBackend::didReceiveStreamMessage(IPC::StreamServerConnectionBase& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::RemoteRenderingBackend::CreateImageBuffer::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::CreateImageBuffer>(connection.connection(), decoder, this, &RemoteRenderingBackend::createImageBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::DestroyGetPixelBufferSharedMemory::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::DestroyGetPixelBufferSharedMemory>(connection.connection(), decoder, this, &RemoteRenderingBackend::destroyGetPixelBufferSharedMemory);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::PutPixelBufferForImageBuffer::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::PutPixelBufferForImageBuffer>(connection.connection(), decoder, this, &RemoteRenderingBackend::putPixelBufferForImageBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::CacheNativeImage::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::CacheNativeImage>(connection.connection(), decoder, this, &RemoteRenderingBackend::cacheNativeImage);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::CacheFont::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::CacheFont>(connection.connection(), decoder, this, &RemoteRenderingBackend::cacheFont);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::DeleteAllFonts::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::DeleteAllFonts>(connection.connection(), decoder, this, &RemoteRenderingBackend::deleteAllFonts);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::ReleaseRemoteResource::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::ReleaseRemoteResource>(connection.connection(), decoder, this, &RemoteRenderingBackend::releaseRemoteResource);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::FinalizeRenderingUpdate::name())
        return IPC::handleMessage<Messages::RemoteRenderingBackend::FinalizeRenderingUpdate>(connection.connection(), decoder, this, &RemoteRenderingBackend::finalizeRenderingUpdate);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::GetPixelBufferForImageBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::GetPixelBufferForImageBuffer>(connection, decoder, this, &RemoteRenderingBackend::getPixelBufferForImageBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::GetPixelBufferForImageBufferWithNewMemory::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::GetPixelBufferForImageBufferWithNewMemory>(connection, decoder, this, &RemoteRenderingBackend::getPixelBufferForImageBufferWithNewMemory);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::GetDataURLForImageBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::GetDataURLForImageBuffer>(connection, decoder, this, &RemoteRenderingBackend::getDataURLForImageBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::GetDataForImageBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::GetDataForImageBuffer>(connection, decoder, this, &RemoteRenderingBackend::getDataForImageBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::GetShareableBitmapForImageBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::GetShareableBitmapForImageBuffer>(connection, decoder, this, &RemoteRenderingBackend::getShareableBitmapForImageBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::GetFilteredImageForImageBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::GetFilteredImageForImageBuffer>(connection, decoder, this, &RemoteRenderingBackend::getFilteredImageForImageBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::MarkSurfaceNonVolatile::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::MarkSurfaceNonVolatile>(connection, decoder, this, &RemoteRenderingBackend::markSurfaceNonVolatile);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::SwapToValidFrontBuffer::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::SwapToValidFrontBuffer>(connection, decoder, this, &RemoteRenderingBackend::swapToValidFrontBuffer);
    if (decoder.messageName() == Messages::RemoteRenderingBackend::MarkSurfacesVolatile::name())
        return IPC::handleMessageSynchronous<Messages::RemoteRenderingBackend::MarkSurfacesVolatile>(connection, decoder, this, &RemoteRenderingBackend::markSurfacesVolatile);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(connection);
#if ENABLE(IPC_TESTING_API)
    if (connection.connection().ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled stream message %s to %" PRIu64, IPC::description(decoder.messageName()), decoder.destinationID());
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS)
